/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.factory.tile;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Random;
import java.util.function.Predicate;
import java.util.stream.Stream;

import org.apache.commons.lang3.tuple.Pair;

import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EntitySelectors;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;

import net.minecraftforge.common.capabilities.ICapabilityProvider;

import buildcraft.api.core.EnumPipePart;
import buildcraft.api.mj.MjAPI;
import buildcraft.api.mj.MjBattery;
import buildcraft.api.mj.MjCapabilityHelper;
import buildcraft.api.tiles.IDebuggable;

import buildcraft.lib.block.BlockBCBase_Neptune;
import buildcraft.lib.inventory.ItemTransactorHelper;
import buildcraft.lib.inventory.NoSpaceTransactor;
import buildcraft.lib.inventory.TransactorEntityItem;
import buildcraft.lib.misc.AdvancementUtil;
import buildcraft.lib.misc.BoundingBoxUtil;
import buildcraft.lib.mj.MjBatteryReceiver;
import buildcraft.lib.tile.TileBC_Neptune;
import buildcraft.lib.tile.item.ItemHandlerManager.EnumAccess;
import buildcraft.lib.tile.item.ItemHandlerSimple;

import buildcraft.factory.block.BlockChute;

public class TileChute extends TileBC_Neptune implements ITickable, IDebuggable {
    private static final ResourceLocation ADVANCEMENT_DID_INSERT = new ResourceLocation("buildcraftfactory:retired_hopper");

    private static final int PICKUP_MAX = 3;

    public final ItemHandlerSimple inv = itemManager.addInvHandler(
        "inv",
        4,
        EnumAccess.INSERT,
        EnumPipePart.VALUES
    );

    private final MjBattery battery = new MjBattery(1 * MjAPI.MJ);
    private int progress = 0;

    public TileChute() {
        caps.addProvider(new MjCapabilityHelper(new MjBatteryReceiver(battery)));
    }

    public static boolean hasInventoryAtPosition(IBlockAccess world, BlockPos pos, EnumFacing side) {
        TileEntity tile = world.func_175625_s(pos);
        return ItemTransactorHelper.getTransactor(tile, side.func_176734_d()) != NoSpaceTransactor.INSTANCE;
    }

    private void pickupItems(EnumFacing currentSide) {
        AxisAlignedBB aabb = BoundingBoxUtil.extrudeFace(func_174877_v(), currentSide, 0.25);
        int count = PICKUP_MAX;
        for (EntityItem entity : field_145850_b.func_175647_a(EntityItem.class, aabb, EntitySelectors.field_94557_a)) {
            int moved = ItemTransactorHelper.move(new TransactorEntityItem(entity), inv, count);
            count -= moved;
            if (count <= 0) {
                return;
            }
        }
    }

    private void putInNearInventories(EnumFacing currentSide) {
        boolean[] didWork = { false };
        List<EnumFacing> sides = new ArrayList<>(Arrays.asList(EnumFacing.field_82609_l));
        Collections.shuffle(sides, new Random());
        sides.removeIf(Predicate.isEqual(currentSide));
        Stream.<Pair<EnumFacing, ICapabilityProvider>>concat(
            sides.stream()
                .map(side -> Pair.of(side, field_145850_b.func_175625_s(field_174879_c.func_177972_a(side)))),
            sides.stream()
                .flatMap(side ->
                    field_145850_b.func_72872_a(Entity.class, new AxisAlignedBB(field_174879_c.func_177972_a(side))).stream()
                        .filter(entity -> !(entity instanceof EntityLivingBase))
                        .map(entity -> Pair.of(side, entity))
                )
        )
            .map(sideProvider -> ItemTransactorHelper.getTransactor(sideProvider.getRight(), sideProvider.getLeft().func_176734_d()))
            .filter(Predicate.isEqual(NoSpaceTransactor.INSTANCE).negate())
            .forEach(transactor -> {
                if (ItemTransactorHelper.move(inv, transactor, 1) > 0) {
                    didWork[0] = true;
                }
            });
        if (didWork[0]) {
            AdvancementUtil.unlockAdvancement(getOwner().getId(), ADVANCEMENT_DID_INSERT);
        }
    }

    // ITickable

    @Override
    public void func_73660_a() {
        if (field_145850_b.field_72995_K) {
            return;
        }

        if (!(field_145850_b.func_180495_p(field_174879_c).func_177230_c() instanceof BlockChute)) {
            return;
        }

        battery.tick(func_145831_w(), func_174877_v());

        EnumFacing currentSide = field_145850_b.func_180495_p(field_174879_c).func_177229_b(BlockBCBase_Neptune.BLOCK_FACING_6);

        int target = 100000;
        if (currentSide == EnumFacing.UP) {
            progress += 1000; // can be free because of gravity
        }
        progress += battery.extractPower(0, target - progress);

        if (progress >= target) {
            progress = 0;
            pickupItems(currentSide);
        }

        putInNearInventories(currentSide);
    }

    @Override
    public void func_145839_a(NBTTagCompound nbt) {
        super.func_145839_a(nbt);
        progress = nbt.func_74762_e("progress");
        battery.deserializeNBT(nbt.func_74775_l("battery"));
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound nbt) {
        super.func_189515_b(nbt);
        nbt.func_74768_a("progress", progress);
        nbt.func_74782_a("battery", battery.serializeNBT());
        return nbt;
    }

    // IDebuggable

    @Override
    public void getDebugInfo(List<String> left, List<String> right, EnumFacing side) {
        left.add("battery = " + battery.getDebugString());
        left.add("progress = " + progress);
    }
}
